use crate::{add_train::Error, Display, Options};

use super::add_train::{add_train, TrainOffsets, WindowOffsets};

/// Adds a C51 train to the display.
///
/// # Arguments
///
/// * `x` - The x-coordinate where the train should be added.
/// * `names` - A slice of strings representing the names to be displayed.
/// * `display` - The display where the train will be added.
/// * `options` - Options for customizing the train.
///
/// # Returns
///
/// * `Result<(), Error>` - Returns `Ok(())` if successful, otherwise returns an `Error`.
pub fn add_c51<T: AsRef<str>, U: Display, V: Options>(
    x: i32,
    names: &[T],
    display: &U,
    options: &V,
) -> Result<(), Error> {
    const ENGINE: [[&str; 12]; 6] = [
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|=[]=-      ||      ||      |  ||=======_|__",
            "/~\\____|___|/~\\_|  O=======O=======O   |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|=[]=- O=======O=======O    |  ||=======_|__",
            "/~\\____|___|/~\\_|      ||      ||      |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|==[]=- O=======O=======O   |  ||=======_|__",
            "/~\\____|___|/~\\_|      ||      ||      |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|===[]=- O=======O=======O  |  ||=======_|__",
            "/~\\____|___|/~\\_|      ||      ||      |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|===[]=-    ||      ||      |  ||=======_|__",
            "/~\\____|___|/~\\_|    O=======O=======O |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
        [
            "        ___                                            ",
            "       _|_|_  _     __       __             ___________",
            "    D__/   \\_(_)___|  |__H__|  |_____I_Ii_()|_________|",
            "     | `---'   |:: `--'  H  `--'         |  |___ ___|  ",
            "    +|~~~~~~~~++::~~~~~~~H~~+=====+~~~~~~|~~||_| |_||  ",
            "    ||        | ::       H  +=====+      |  |::  ...|  ",
            "|    | _______|_::-----------------[][]-----|       |  ",
            "| /~~ ||   |-----/~~~~\\  /[I_____I][][] --|||_______|__",
            "------'|oOo|==[]=-     ||      ||      |  ||=======_|__",
            "/~\\____|___|/~\\_|   O=======O=======O  |__|+-/~\\_|     ",
            "\\_/         \\_/  \\____/  \\____/  \\____/      \\_/       ",
            "                                                       ",
        ],
    ];

    const COAL: [&str; 12] = [
        "                              ",
        "                              ",
        "                              ",
        "    _________________         ",
        "   _|                \\_____A  ",
        " =|                        |  ",
        " -|                        |  ",
        "__|________________________|_ ",
        "|__________________________|_ ",
        "   |_D__D__D_|  |_D__D__D_|   ",
        "    \\_/   \\_/    \\_/   \\_/    ",
        "                              ",
    ];

    const CAR: [&str; 12] = [
        "                              ",
        "  __________________________  ",
        "  |   ___  ___  ___  ___   |   ",
        "  |   | |  | |  | |  | |   |  ",
        "  |   |_|  |_|  |_|  |_|   |  ",
        "  |                        |  ",
        "  | {} |  ",
        "__|________________________|_ ",
        "|__________________________|_ ",
        "   |_D__D__D_|  |_D__D__D_|   ",
        "    \\_/   \\_/    \\_/   \\_/  ",
        "                              ",
    ];

    const OFFSETS: TrainOffsets<2, 4> = TrainOffsets {
        funnel: 7,
        engine_windows: WindowOffsets {
            height: 3,
            window_positions: [45, 49],
        },
        car_windows: WindowOffsets {
            height: 2,
            window_positions: [4, 9, 14, 19],
        },
        car_text_width: 22,
    };

    add_train(x, &ENGINE, &COAL, &CAR, OFFSETS, names, display, options)
}
